.TH ETI 4 "March 13, 1993" "San Francisco State University"
.\" Copyright 1993 by Eric P. Scott.  All Rights Reserved.
.SH NAME
etI \- EtherTalk Phase 1 Protocol Access
.SH SYNOPSIS
#include <sys/types.h>
.br
#include <sys/ioctl.h>
.br
#include <sys/socket.h>
.br
#include <net/if.h>
.br
#include <net/etherdefs.h>
.SH DESCRIPTION
.if t .ad b
The
.I etI?
special files provide link-level access to the Ethernet
driver for EtherTalk Phase 1 and AARP frames.  They do not
implement AppleTalk protocols in the kernel.
.PP
Each real Ethernet interface has a corresponding minor device.
The exact association depends on the order interfaces attached.
It is possible to determine the name of the hardware interface
associated with an open minor device. However, on a machine
with exactly one Ethernet interface, this is generally
unnecessary.
.PP
Opens are exclusive; an EBUSY error indicates another process'
ownership.
.PP
Standard
.IR read (2)
and
.IR write (2)
(or
.IR readv (2)
and
.IR writev (2))
calls are used to receive and send datagrams. Since
.I recvfrom
and
.I sendto
are not applicable to character special files, the 14-octet
Ethernet header is delivered inline preceding protocol-level
data.
.PP
Read and write operations are atomic; each I/O request attempts
to transfer exactly one datagram. If a datagram is too long to
fit in a supplied
.I read
buffer, excess bytes are discarded. Reads block if no datagrams are
immediately available unless the file descriptor has been marked
non-blocking (see
.IR ioctl (2)),
in which case EWOULDBLOCK is returned.
.PP
The
.IR select (2)
call may be used to determine when more data arrives.
The device also supports asynchronous I/O in the same manner as
sockets.
.PP
If datagrams arrive faster than the program accepts them,
most-recently received datagrams will be discarded once the
device's input queue fills. The queue size is normally
IFQ_MAXLEN (defined in <net/if.h>), but can be changed by
patching the driver variable
.BR et1qmaxlen .
.PP
Write operations require the destination address and Ethernet
type be set as desired. The source address is ignored, as the
Ethernet driver supplies an appropriate value. If the supplied
message is too large to fit in an Ethernet datagram, the error
EMSGSIZE is returned, and no datagram is transmitted. If the
message is too small to contain a complete Ethernet header, the
error EINVAL is returned, and no datagram is transmitted. If
the message is smaller than ETHERMIN, the Ethernet driver may
append
.if n "garbage"
.if t ``garbage''
to the end of the transmitted datagram. If the Ethernet type
field is set to a value the receive filter would refuse, the
error EADDRNOTAVAIL is returned and no datagram is transmitted.
.PP
There is no guarantee that outgoing datagrams will actually be
delivered. The
.I write
call will return any error code returned by the interface
output routine.
.PP
Non-blocking I/O is not supported for write operations.
.IR Select (2)
always indicates the device is ready to accept data.
.PP
Four
.I ioctl
operations normally used with sockets are supported:
SIOCGIFADDR, SIOCGIFBRDADDR, SIOCGIFFLAGS, and SIOCGIFMTU.
All take the expected
.B struct ifreq
argument, but
.B ifr_name
is not examined, and its contents are overwritten with the name
of the associated interface. SIOCGIFADDR and SIOCGIFBRDADDR
format their
.B struct sockaddr
as
.PP
.TS
center box;
ce|ce
c c
c c
c c
c c
c|c.
family	dhost
_	\^
\&	\^
_
shost	\&
\^	_
\^	type
.TE
.PP
Both set family to AF_UNSPEC. SIOCGIFADDR sets shost to the
Ethernet's hardware address. SIOCGIFBRDADDR sets dhost to the
value of the kernel variable
.BR etherbroadcastaddr .
.PP
The driver may be compiled to include additional support for
etIE? and etIA? devices.  These function identically to etI?,
except that etIE? handles strictly EtherTalk Phase 1 (type
0x809b), and etIA? strictly AARP (type 0x80f3).  SIOCGIFADDR
and SIOCGIFBRDADDR, when applied to these devices, additionally
set type to the corresponding value.  The notion of
.if n "exclusive open"
.if t ``exclusive open''
is extended to encompass protocol access; e.g., etI? monopolizes
the interface, while etIE? and etIA? can be opened separately,
yet either will prevent etI? from being opened.
.SH ERRORS
An I/O operation may fail with one of the following errors returned:
.TP
[ENXIO]
when trying to open an unused minor device, or when the driver
has been unloaded or is in the midst of being unloaded;
.TP
[EBUSY]
when trying to open an already open minor device;
.TP
[EINVAL]
when attempting to write fewer than 14 bytes;
.TP
[EMSGSIZE]
when attempting to write more than 1514 bytes;
.TP
[ENOBUFS]
when a kernel buffer can not be allocated for data to be written;
.TP
[EADDRNOTAVAIL]
when attempting to write an unauthorized Ethernet type;
.TP
[ESRCH]
when attempting to specify a nonexistent process to receive
SIGIO signals;
.TP
[EPERM]
when attempting to specify a system process or process group and
the caller is not the superuser;
.TP
[ENOTTY]
when attempting to perform an unsupported
.IR ioctl (2)
operation, including those specific to terminal devices;
.TP
[EINVAL]
when attempting to perform an unsupported
.IR ioctl (2)
operation, including those specific to sockets;
.PP
as well as any errors described in individual I/O calls'
manual pages.
.SH FILES
/dev/etI?
.br
/dev/etIE?
.br
/dev/etIA?
.SH AUTHOR
Eric P. Scott
.SH "SEE ALSO"
read(2), write(2), select(2), fcntl(2), ioctl(2), intro(4N), arp(4P)
.SH DIAGNOSTICS
.B "venet1_init: requested major device number unavailable."
The server's Load Commands specify the use of a major device
number currently in use by another character special device, or
outside the range of
.BR cdevsw .
.PP
\f3ET1 protocol access enabled for interface %s%d, type "%s".\fP
The driver successfully initialized itself and instantiated a
minor device for a real interface.
.PP
.B "ET1 protocol access disabled for interface %s%d."
The driver successfully freed all resources associated with the
named interface.
.PP
.B "ET1 protocol access was open for interface %s%d!"
A request was made for the server to unload, but the driver was
currently being used.  The offending process will be given an
opportunity to get its claws out of kernel memory about to be
unmapped.  With luck, the system won't panic a few seconds
later.
.SH BUGS
There is no way to
.if n "peek"
.if t ``peek''
at incoming data.
.PP
There's currently no way to access interface statistics, such as
the number of datagrams dropped from the input queue.

